@@ -432,10 +432,31 @@ gcm_calib_start_idle_cb (gpointer user_data)
 
 	/* actuall do the action */
 	calib->started_calibration = TRUE;
-	ret = gcm_calibrate_display (calib->calibrate,
-				     calib->device,
-				     calib->main_window,
-				     &error);
+
+	switch (calib->device_kind) {
+	case CD_DEVICE_KIND_CAMERA:
+	case CD_DEVICE_KIND_WEBCAM:
+		ret = gcm_calibrate_device (calib->calibrate,
+					    calib->device,
+					    calib->main_window,
+					    &error);
+		break;
+	case CD_DEVICE_KIND_DISPLAY:
+		ret = gcm_calibrate_display (calib->calibrate,
+					     calib->device,
+					     calib->main_window,
+					     &error);
+		break;
+	case CD_DEVICE_KIND_PRINTER:
+		ret = gcm_calibrate_printer (calib->calibrate,
+					     calib->device,
+					     calib->main_window,
+					     &error);
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
 	if (!ret) {
 		gcm_calibrate_set_title (calib->calibrate, _("Failed to calibrate"));
 		gcm_calibrate_set_message (calib->calibrate, error->message);
@@ -1162,6 +1183,9 @@ gcm_calib_reference_kind_combobox_cb (GtkComboBox *combo_box,
 
 	/* not sorted so we can just use the index */
 	reference_kind = gtk_combo_box_get_active (GTK_COMBO_BOX (combo_box));
+	g_object_set (calib->calibrate,
+		      "reference-kind", reference_kind,
+		      NULL);
 	filename = gcm_calib_reference_kind_to_image_filename (reference_kind);
 
 	/* fallback */
